@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Module providing the `Pool` class for managing a process pool
 #
@@ -7,7 +6,6 @@
 # Copyright (c) 2006-2008, R Oudkerk
 # Licensed to PSF under a Contributor Agreement.
 #
-from __future__ import absolute_import
 
 #
 # Imports
@@ -43,7 +41,8 @@ from .exceptions import (
     TimeoutError,
     WorkerLostError,
 )
-from .five import Empty, Queue, range, values, reraise, monotonic
+from time import monotonic
+from queue import Queue, Empty
 from .util import Finalize, debug, warning
 
 MAXMEM_USED_FMT = """\
@@ -211,7 +210,7 @@ class MaybeEncodingError(Exception):
     def __init__(self, exc, value):
         self.exc = repr(exc)
         self.value = repr(value)
-        super(MaybeEncodingError, self).__init__(self.exc, self.value)
+        super().__init__(self.exc, self.value)
 
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, str(self))
@@ -233,7 +232,7 @@ def soft_timeout_sighandler(signum, frame):
 #
 
 
-class Worker(object):
+class Worker:
 
     def __init__(self, inq, outq, synq=None, initializer=None, initargs=(),
                  maxtasks=None, sentinel=None, on_exit=None,
@@ -536,7 +535,7 @@ class Supervisor(PoolThread):
 
     def __init__(self, pool):
         self.pool = pool
-        super(Supervisor, self).__init__()
+        super().__init__()
 
     def body(self):
         debug('worker handler starting')
@@ -556,8 +555,8 @@ class Supervisor(PoolThread):
                     pool._maintain_pool()
                     time.sleep(0.1)
 
-            # Keep maintaing workers until the cache gets drained, unless
-            # the pool is termianted
+            # Keep maintaining workers until the cache gets drained, unless
+            # the pool is terminated
             pool.restart_state = prev_state
             while self._state == RUN and pool._state == RUN:
                 pool._maintain_pool()
@@ -577,7 +576,7 @@ class TaskHandler(PoolThread):
         self.outqueue = outqueue
         self.pool = pool
         self.cache = cache
-        super(TaskHandler, self).__init__()
+        super().__init__()
 
     def body(self):
         cache = self.cache
@@ -652,7 +651,7 @@ class TimeoutHandler(PoolThread):
         self.t_soft = t_soft
         self.t_hard = t_hard
         self._it = None
-        super(TimeoutHandler, self).__init__()
+        super().__init__()
 
     def _process_by_pid(self, pid):
         return next((
@@ -793,7 +792,7 @@ class ResultHandler(PoolThread):
         self.on_job_ready = on_job_ready
         self.on_ready_counters = on_ready_counters
         self._make_methods()
-        super(ResultHandler, self).__init__()
+        super().__init__()
 
     def on_stop_not_started(self):
         # used when pool started without result handler thread.
@@ -960,7 +959,7 @@ class ResultHandler(PoolThread):
               len(cache), self._state)
 
 
-class Pool(object):
+class Pool:
     '''
     Class which supports an async version of applying functions to arguments.
     '''
@@ -1240,7 +1239,7 @@ class Pool(object):
                     elif sched_for and not sched_for._is_alive():
                         self.on_job_process_down(job, sched_for.pid)
 
-            for worker in values(cleaned):
+            for worker in cleaned.values():
                 if self.on_process_down:
                     if not shutdown:
                         self._process_cleanup_queues(worker)
@@ -1263,8 +1262,8 @@ class Pool(object):
     def mark_as_worker_lost(self, job, exitcode):
         try:
             raise WorkerLostError(
-                'Worker exited prematurely: {0}.'.format(
-                    human_status(exitcode)),
+                'Worker exited prematurely: {0} Job: {1}.'.format(
+                    human_status(exitcode), job._job),
             )
         except WorkerLostError:
             job._set(None, (False, ExceptionInfo()))
@@ -1308,7 +1307,7 @@ class Pool(object):
                 yield worker
 
     def _worker_active(self, worker):
-        for job in values(self._cache):
+        for job in self._cache.values():
             if worker.pid in job.worker_pids():
                 return True
         return False
@@ -1355,9 +1354,7 @@ class Pool(object):
                 raise
             except OSError as exc:
                 if get_errno(exc) == errno.ENOMEM:
-                    reraise(MemoryError,
-                            MemoryError(str(exc)),
-                            sys.exc_info()[2])
+                    raise MemoryError from exc
                 raise
 
     def _setup_queues(self):
@@ -1628,7 +1625,7 @@ class Pool(object):
         debug('pool join complete')
 
     def restart(self):
-        for e in values(self._poolctrl):
+        for e in self._poolctrl.values():
             e.set()
 
     @staticmethod
@@ -1694,8 +1691,10 @@ class Pool(object):
                         p.join()
             debug('pool workers joined')
 
-        inqueue.close()
-        outqueue.close()
+        if inqueue:
+            inqueue.close()
+        if outqueue:
+            outqueue.close()
 
     @property
     def process_sentinels(self):
@@ -1706,7 +1705,7 @@ class Pool(object):
 #
 
 
-class ApplyResult(object):
+class ApplyResult:
     _worker_lost = None
     _write_to = None
     _scheduled_for = None
@@ -1748,7 +1747,7 @@ class ApplyResult(object):
         )
 
     def ready(self):
-        return self._event.isSet()
+        return self._event.is_set()
 
     def accepted(self):
         return self._accepted
@@ -1925,7 +1924,7 @@ class MapResult(ApplyResult):
 #
 
 
-class IMapIterator(object):
+class IMapIterator:
     _worker_lost = None
 
     def __init__(self, cache, lost_worker_timeout=LOST_WORKER_TIMEOUT):
